// Copyright (c) 2023 Dominic Masters
//
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SphereMesh.hpp"

using namespace Dawn;

void SphereMesh::createSphere(
  Mesh *mesh,
  float_t radius,
  int32_t slices,
  int32_t stacks
) {
  std::vector<glm::vec3> positions;

  // Create vertices
  int32_t c = 0;
  for (int32_t i = 0; i <= stacks; i++) {
    float_t phi = MATH_PI * i / stacks;
    float_t cosPhi = cos(phi);
    float_t sinPhi = sin(phi);

    for (int32_t j = 0; j <= slices; j++) {
      float_t theta = 2 * MATH_PI * j / slices;
      float_t cosTheta = cos(theta);
      float_t sinTheta = sin(theta);

      glm::vec3 v;
      v.x = radius * sinPhi * cosTheta;
      v.y = radius * sinPhi * sinTheta;
      v.z = radius * cosPhi;
      
      positions.push_back(v);
    }
  }

  // Create indices
  std::vector<meshindice_t> indices;
  for (int32_t i = 0; i < stacks; i++) {
    for (int32_t j = 0; j < slices; j++) {
      meshindice_t p1 = i * (slices + 1) + j;
      meshindice_t p2 = p1 + slices + 1;

      indices.push_back(p1);
      indices.push_back(p2);
      indices.push_back(p1 + 1);

      indices.push_back(p1 + 1);
      indices.push_back(p2);
      indices.push_back(p2 + 1);
    }
  }

  mesh->createBuffers(positions.size(), indices.size());
  mesh->bufferPositions(0, positions.data(), positions.size());
  mesh->bufferIndices(0, indices.data(), indices.size());
}